<?php
/**
 * This file is part of Advanced Custom Fields: Grid.
 *
 * Advanced Custom Fields: Grid is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 2 of the License, or any later version.
 *
 * Advanced Custom Fields: Grid is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Advanced Custom Fields: Grid.  If not, see <https://www.gnu.org/licenses/>.
 */

declare( strict_types=1 );

namespace AcfGrid\Grid;

/**
 * Gutenberg block for ACF Grid.
 *
 * @since 20220811
 */
class Block {
	private const SIMPLE_LIGHTBOX_HANDLE = 'acf-grid-simple-lightbox';

	/** @var string Path to plugin’s root file. */
	private string $plugin_file;

	/**
	 * Block constructor.
	 *
	 * @param string $plugin_file The path to plugin’s root file.
	 *
	 * @since 20220811
	 */
	public function __construct( string $plugin_file ) {
		$this->plugin_file = $plugin_file;
		add_action( 'init', [ $this, 'register' ] );
	}

	/**
	 * Registers the block to Gutenberg.
	 *
	 * @return void
	 *
	 * @since 20220811
	 */
	public function register() { // phpcs:ignore NeutronStandard.Functions.LongFunction
		$editor_asset = include plugin_dir_path( $this->plugin_file ) . 'build/index.asset.php';
		$editor_handle = 'acf-grid-block-editor';
		wp_register_script(
			$editor_handle,
			plugins_url( 'build/index.js', $this->plugin_file ),
			$editor_asset['dependencies'],
			$editor_asset['version'],
			true
		);

		wp_set_script_translations(
			$editor_handle,
			'acf-grid',
			plugin_dir_path( $this->plugin_file ) . 'languages'
		);

		wp_register_style(
			'acf-grid-block-style',
			plugins_url( 'build/style-index.css', $this->plugin_file ),
			[],
			$editor_asset['version']
		);

		$view_asset = include plugin_dir_path( $this->plugin_file ) . 'build/view.asset.php';
		wp_register_style(
			self::SIMPLE_LIGHTBOX_HANDLE,
			plugins_url( 'build/view.css', $this->plugin_file ),
			[],
			$view_asset['version']
		);

		wp_register_script(
			self::SIMPLE_LIGHTBOX_HANDLE,
			plugins_url( 'build/view.js', $this->plugin_file ),
			$view_asset['dependencies'],
			$view_asset['version'],
			true
		);

		register_block_type( __DIR__, [
			'render_callback' => [ $this, 'render' ],
		] );
	}

	/**
	 * Renders the block’s content to show on the front-end.
	 *
	 * @param array $block_attributes The block attributes defined in the editor.
	 *
	 * @return string The block content.
	 *
	 * @since 20220811
	 */
	public function render( array $block_attributes ): string { // phpcs:ignore NeutronStandard.Functions.LongFunction
		$gutter = $block_attributes['gutter'];
		$classes = [
			'acf-grid-desktop-' . $block_attributes['desktopColumns'],
			'acf-grid-tablet-' . $block_attributes['tabletColumns'],
			'acf-grid-mobile-' . $block_attributes['mobileColumns'],
		];
		$wrapper_attributes = get_block_wrapper_attributes( [
			'class' => implode( ' ', $classes ),
			'style' => sprintf( '--acf-grid-gutter: %dpx', $gutter ),
		] );

		$link_attributes = '';
		if ( $block_attributes['useLightbox'] ?? false ) {
			wp_enqueue_style( self::SIMPLE_LIGHTBOX_HANDLE );
			wp_enqueue_script( self::SIMPLE_LIGHTBOX_HANDLE );

			if ( $block_attributes['lightboxGallery'] ?? false ) {
				$gallery = 'gallery-' . esc_attr( uniqid() );
				$link_attributes .= sprintf( ' rel="%1$s"', $gallery );
				$wrapper_attributes .= sprintf( ' data-lightbox="%1$s"', $gallery );
			} else {
				$link_attributes = ' rel="lightbox"';
			}
		}

		if ( strpos( $block_attributes['className'] ?? '', 'is-style-masonry' ) !== false ) {
			wp_enqueue_script( 'jquery-masonry' );
			/** @noinspection JSUnresolvedFunction */
			wp_add_inline_script( 'jquery-masonry', <<<"JS"
var grid = jQuery('.wp-block-tandem-acf-grid.is-style-masonry').imagesLoaded(function() {
	grid.masonry({
		itemSelector: 'a',
		gutter: $gutter,
		percentPosition: true
	});
});
JS
			);
		}

		$field = $block_attributes['fieldName'];
		$content = '';
		/** @noinspection PhpUndefinedFunctionInspection */
		$images = get_field( $field, get_the_ID() );
		if ( ! $images ) {
			return '';
		}
		foreach ( $images as $image ) {
			/** @noinspection HtmlUnknownTarget */
			/** @noinspection HtmlUnknownAttribute */
			$content .= sprintf(
				'<a href="%s" title="%s" %s><img src="%s" alt="%s"/></a>',
				esc_url( $image['url'] ),
				esc_attr( $image['alt'] ),
				$link_attributes,
				esc_url( $image['sizes']['large'] ),
				esc_attr( $image['alt'] )
			);
		}

		/** @noinspection HtmlUnknownAttribute */
		return sprintf( '<div %s>%s</div>', $wrapper_attributes, $content );
	}
}
